/* -*- c++ -*- */
/* 
 * Copyright 2016 <+YOU OR YOUR COMPANY+>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "foo_source_impl.h"

namespace gr {
    namespace foobar {

        const pmt::pmt_t foo_source_impl::INPUT_PORT = pmt::string_to_symbol("pdus");
        const pmt::pmt_t foo_source_impl::OUTPUT_PORT = pmt::string_to_symbol("pdus");

        foo_source::sptr
            foo_source::make()
            {
                return gnuradio::get_initial_sptr
                    (new foo_source_impl());
            }

        /*
         * The private constructor
         */
        foo_source_impl::foo_source_impl()
            : gr::sync_block("foo_source",
                    gr::io_signature::make(0, 0, 0),
                    gr::io_signature::make(0, 0, 0))
        {
            this->stopThread = false;

            message_port_register_out(OUTPUT_PORT);
            message_port_register_in(INPUT_PORT);
            set_msg_handler(INPUT_PORT, boost::bind(&foo_source_impl::handler, this, _1));

            this->thread = boost::thread(boost::bind(&foo_source_impl::maker, this));
        }

        /*
         * Our virtual destructor.
         */
        foo_source_impl::~foo_source_impl()
        {
            this->stopThread = true;
            this->thread.join();
        }

        void foo_source_impl::handler(pmt::pmt_t pdu){
            this->requestsLock.lock();
            this->requests.push_back(pmt::to_uint64(pmt::dict_ref(pdu, pmt::mp("bytes"), pmt::from_uint64(0))));
            this->requestsLock.unlock();
        }

        void foo_source_impl::maker(){
            uint64_t byteCount;
            const pmt::pmt_t data = pmt::cons(pmt::make_dict(), pmt::make_u8vector(1, 0x7e));

            while(! this->stopThread){
                byteCount = 0LU;

                this->requestsLock.lock();
                if(! this->requests.empty()){
                    byteCount = this->requests[0];
                    this->requests.pop_back();
                }
                this->requestsLock.unlock();
    
                if(byteCount != 0){
                    for(uint64_t a = 0; a < byteCount; a++){
                        message_port_pub(OUTPUT_PORT, data);
                        boost::this_thread::sleep_for(boost::chrono::milliseconds(10));
                    }   
                }else{
                    boost::this_thread::sleep_for(boost::chrono::milliseconds(10));
                }
            }
        }

        int
            foo_source_impl::work(int noutput_items,
                    gr_vector_const_void_star &input_items,
                    gr_vector_void_star &output_items)
            {
                return noutput_items;
            }

    } /* namespace foobar */
} /* namespace gr */

