#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 <+YOU OR YOUR COMPANY+>.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

import numpy
import time
import pmt
from gnuradio import gr, gr_unittest
from gnuradio import blocks
import foobar_swig as foobar

# produces pdus when it gets input to its streaming byte input
# only here because there is no way (that i know of) to inject
# pdus to the message ports of the block under test.  the vector
# source will trigger the work function, and then the work function
# will use the normal means to send out two pdus.
class producer(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(self, name='producer', in_sig=[numpy.byte], out_sig=[])
        # register the output pmt port for this block
        self.message_port_register_out(pmt.intern('pdus'))

    def work(self, input_items, output_items):
        # ever time this function is called, send out two pdus
        pdu = pmt.make_dict()

        pdu = pmt.dict_add(pdu, pmt.intern('bytes'), pmt.from_uint64(3))
        self.message_port_pub(pmt.intern('pdus'), pdu)

        # stall for one second just because
        time.sleep(1)

        pdu = pmt.dict_add(pdu, pmt.intern('bytes'), pmt.from_uint64(5))
        self.message_port_pub(pmt.intern('pdus'), pdu)
        
        return len(input_items[0])

# consumes the pdus output by the block under test.  in this case it should get
# 8 pdus.  for each pdu read in, the receivedVectors counter will be incremented
# by however meany elements are in the vector.   there are no streaming ports for 
# this block.
class consumer(gr.sync_block):
    def __init__(self, parent):
        gr.sync_block.__init__(self, name='consumer', in_sig=[], out_sig=[])
        
        # this holds the reference to receivedVectors
        self.parent = parent
        # register the input port and set the handler
        self.message_port_register_in(pmt.intern('pdus'))
        self.set_msg_handler(pmt.intern('pdus'), self.handler)

    # increments the counter for each element in the vector that was read in
    def handler(self, pdu):
        self.parent.receivedVectors += pmt.length(pmt.cdr(pdu))
        print 'got data %d' % self.parent.receivedVectors


class qa_foo_source (gr_unittest.TestCase):
    def setUp (self):
        self.tb = gr.top_block ()
        # this is a place to keep the number of pdus that were received by the consumer
        self.receivedVectors = 0

    def tearDown (self):
        print 'called teardown'
        self.tb = None

    def test_001_t (self):
        # a place to keep track of how many PDUs were received
        self.receivedVectors = 0
        # new instane of the message producer
        prod = producer()
        # new instance of the message consumer
        cons = consumer(self)
        # new instance of a vector source that sends out a single byte once
        vec = blocks.vector_source_b([1,], False)
        # new instance of the block to be tested
        foo = foobar.foo_source()

        # connect the vector source to the message producer
        self.tb.connect(vec, prod)
        # connect the pdus message output of the producer to the pdus message input
        # of the block to be tested
        self.tb.msg_connect(prod, pmt.intern('pdus'), foo, pmt.intern('pdus'))
        # connect the pdus message output of the block to be tested to the pdus message
        # input of the consumer
        self.tb.msg_connect(foo, pmt.intern('pdus'), cons, pmt.intern('pdus'))

        # notify that the top block is about to start
        print 'starting'

        # calling run will just cause the entire test to hang after completion.  not
        # even the tearDown method gets called
        #self.tb.run()

        
        # using the start() method will at least cause the test to get all the way to 
        # the tearDown() method, but it stalls there.

        # start the top block
        self.tb.start ()

        # sleep for 5 seconds to allow time for the messages to be sent and processed
        time.sleep(5)
        
        # stop the top block
        self.tb.stop()

        # Notify that the test has completed
        print 'done'

        # print out how many pdus were received
        print '%d' % self.receivedVectors

        self.assertTrue(1 == 1, 'should not have failed')
        
        # uncommenting this line causes the test to end as expected.  leaving it commented
        # will cause the test to simply hang.
        #self.assertTrue(1 == 0, 'failed as it should')


if __name__ == '__main__':
    gr_unittest.run(qa_foo_source, "qa_foo_source.xml")
